#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <rthw.h>
#include <rtthread.h>
#include <rtdevice.h>
#include <finsh.h>

#include "APP_Elog.h"
#include "APP_I2c.h"

#include "APP_GPIODefined.h"
#include "APP_PCM5122_DAC.h"
#include "drv_i2c.h"
#include "APP_Misc_BSP.h"


#if(BOARD_SEL==BD_MX0810_2020_A50)

#define PCM5122LogDebug(...)                 ELOG_APP_Debug("PCM5122-D", __VA_ARGS__)
#define PCM5122LogError(...)                 ELOG_APP_Error("PCM5122-E", __VA_ARGS__)
#define PCM5122LogWarn(...)                 ELOG_APP_Warn("PCM5122-W", __VA_ARGS__)
static struct rt_mutex PCM5122Lock;
static rt_device_t pcm5122_fd = RT_NULL;
void App_PCM5122_Init(void);

void APP_Pcm512x_InterfaceInit(void)
{
    rt_err_t err = RT_EOK;
    pcm5122_fd = rt_device_find(PCM5122_I2C);
    RT_ASSERT(RT_NULL != pcm5122_fd);
    
     rt_mutex_init(&PCM5122Lock,"pcm_lock",RT_IPC_FLAG_FIFO);
    RT_ASSERT(RT_EOK == err);
  
    App_PCM5122_Init();
    
}


uint8_t PCM5122I2cWrite(uint8_t index, uint32_t devAddress, uint32_t regAddress, uint8_t *i2cdata, uint16_t count, uint8_t Flag16bit)
{
    rt_mutex_take(&PCM5122Lock, RT_WAITING_FOREVER);
    APP_I2CEX_OutPCM5122((index));//0-7
    rt_err_t err = i2c_write_reg_16bit(PCM5122_I2C, devAddress, regAddress, i2cdata, count,Flag16bit);
    rt_mutex_release(&PCM5122Lock);
    return err;
}

uint8_t PCM5122I2cRead(uint8_t index, uint32_t devAddress, uint32_t regAddress, uint8_t *i2cdata, uint16_t count, uint8_t Flag16bit)
{
    rt_mutex_take(&PCM5122Lock, RT_WAITING_FOREVER);
    APP_I2CEX_OutPCM5122((index));//0-7
    rt_err_t err = i2c_read_reg_16bit(PCM5122_I2C, devAddress, regAddress, i2cdata, count,Flag16bit);
    rt_mutex_release(&PCM5122Lock);
    return err;
}

uint8_t Pcm5122_IIC_Write(uint8_t index,uint8_t regAddress, uint8_t *buf, uint16_t count)
{
    uint8_t  ret;
    ret=PCM5122I2cWrite(index,PCM5122_ADDR,regAddress, buf,count,0);
    return ret;
}


uint8_t Pcm5122_IIC_Read(uint8_t index,uint8_t regAddress,uint8_t *buf, uint16_t count)
{
    uint8_t  ret;
    ret=PCM5122I2cRead(index,PCM5122_ADDR,regAddress, buf,count,0);
    return ret;
}

// Step length up
int8_t App_PCM5122_SetVolumeStep_Up(uint8_t outport,uint8_t stepup)
{
    uint8_t LeftVolume;
    uint8_t UpVolume;
    uint8_t MuteStatus = 0;  
    uint8_t volumesize;
    uint8_t temp;
   if(stepup>10)
   {
        PCM5122LogDebug("step up err= %d\n",stepup);
        return ERROR_RET;  
   }
    temp=1;
    Pcm5122_IIC_Write(outport,Digital_Volume_Control,&temp,1);//Right channel volume follows left channel setting
    Pcm5122_IIC_Read(outport,Left_Digital_Volume,&LeftVolume,1);//read volume from reg defult=48
    if(LeftVolume==0x00)
    {
        volumesize=(127-LeftVolume*0.5);
        PCM5122LogDebug("Volume exceeds max volume,volume = %d\r\n",volumesize);
        return ERROR_RET;   
    }
    else 
    {
        Pcm5122_IIC_Read(outport,Volume_Mute_Ctrl,&MuteStatus,1);
        if(MuteStatus == 0x11)
        {
            temp=0;
            Pcm5122_IIC_Write(outport,Volume_Mute_Ctrl,&temp,1);
        }
        UpVolume = (LeftVolume-stepup*2); //up a step
        if(LeftVolume<(stepup*2))
        {
           UpVolume=0x00;
            PCM5122LogDebug("maximum volume,volume = %d;\r\n",UpVolume);
        }
        volumesize =127-(UpVolume*0.5); 
        Pcm5122_IIC_Write(outport,Left_Digital_Volume,&UpVolume,1);
        PCM5122LogDebug("Current++ volume is UpVolume=%d,volumesize=%d dB;\r\n",UpVolume,volumesize);  
        return Ok_RET;   
    }

}

// Step length down
int8_t App_PCM5122_SetVolumeStep_Down(uint8_t outport,uint8_t stepdown)
{
    uint8_t LeftVolume;
    uint8_t UpVolume;
    uint8_t MuteStatus = 0;  
    uint8_t volumesize;
    uint8_t temp;
    if(stepdown>10)
    {
        PCM5122LogDebug("step up err= %d\n",stepdown);
        return ERROR_RET;  
    }
    temp=1;
    Pcm5122_IIC_Write(outport,Digital_Volume_Control,&temp,1);//Right channel volume follows left channel setting
    Pcm5122_IIC_Read(outport,Left_Digital_Volume,&LeftVolume,1);//read volume from reg
    if(LeftVolume==0xff)
    {
        volumesize=128-(LeftVolume+1)*0.5;
        PCM5122LogDebug("Volume exceeds min volume,volume = %d\r\n",volumesize);
        return ERROR_RET;   
    }
    else 
    {
        Pcm5122_IIC_Read(outport,Volume_Mute_Ctrl,&MuteStatus,1);
        if(MuteStatus == 0x11)
        {
            temp=0;
            Pcm5122_IIC_Write(outport,Volume_Mute_Ctrl,&temp,1);
        } 
        UpVolume = (LeftVolume+stepdown*2); //up a step
        if((0xff-LeftVolume)<(stepdown*2))
        {
            UpVolume=0xff; 
            PCM5122LogDebug("min volume,volume = %d;\r\n",UpVolume);
        }
        volumesize =128-(UpVolume+1)*0.5; 
        Pcm5122_IIC_Write(outport,Left_Digital_Volume,&UpVolume,1);
        PCM5122LogDebug("Current-- volume is UpVolume=%d,volumesize=%d dB;\r\n",UpVolume,volumesize);  
    
        return Ok_RET;   
    }
}
//value(-80~0)
void App_PCM5122_SetVolumeValue(uint8_t outport,uint8_t value)
{
    uint8_t MuteStatus = 0;  
    uint8_t volumesize;
    uint8_t temp;
    Pcm5122_IIC_Read(outport,Volume_Mute_Ctrl,&MuteStatus,1);
    if(MuteStatus != 0x11)
    {
        temp=0x11;
        Pcm5122_IIC_Write(outport,Volume_Mute_Ctrl,&temp,1);
    } 
  //  volumesize = (127-value)*2; //set a value
    volumesize =48+value*2; //set a value 48 is 0dB
  //  volumesize =48+value; //set a value 48 is 0dB
    temp=1;
    Pcm5122_IIC_Write(outport,Digital_Volume_Control,&temp,1);//Right channel volume follows left channel setting
    Pcm5122_IIC_Write(outport,Left_Digital_Volume,&volumesize,1);
    if(APP_StorageReadAudio_Mute(outport)==0)
    {
    temp=0x00;
    Pcm5122_IIC_Write(outport,Volume_Mute_Ctrl,&temp,1);
    }
    PCM5122LogDebug("volume port=%d,value=%d,volumesize=%d;\r\n",outport,value,volumesize);  
   
}
void App_PCM5122_SetVolumeMute(uint8_t outport,uint8_t mute)
{
    uint8_t VolumeMuteStatus = 0;
    uint8_t temp;
    Pcm5122_IIC_Read(outport,Volume_Mute_Ctrl,&VolumeMuteStatus,1);
    PCM5122LogDebug("mute the volume outport %d,mute %d;\n\r",outport,mute);  
    if(mute)
    {
        temp=0x11;
        if(VolumeMuteStatus!=temp)
        Pcm5122_IIC_Write(outport,Volume_Mute_Ctrl,&temp,1);
    }
   else
   {
        temp=0x00;
        if(VolumeMuteStatus!=temp)
        Pcm5122_IIC_Write(outport,Volume_Mute_Ctrl,&temp,1);
   }
    /*
    if(mute)//mute on
    {
        PCM5122LogDebug("mute the volume outport %d;\n\r",outport);  
        switch (outport)
        {
            case 0:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE1);
            break;
            case 1:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE2);
            break;
            case 2:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE3);
            break;
            case 3:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE4);
            break;
            case 4:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE5);
            break;
            case 5:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE6);
            break;
            case 6:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE7);
            break;
            case 7:
            GPIO_App_Low(gpio_out_PCM5122_AUDIO_MUTE8);
            break;
            default :
            break;
        }     
    }
    else
    {
        PCM5122LogDebug("unmute the volume outport %d;\n\r",outport);
        switch (outport)
        {
            case 0:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE1);
            break;
            case 1:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE2);
            break;
            case 2:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE3);
            break;
            case 3:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE4);
            break;
            case 4:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE5);
            break;
            case 5:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE6);
            break;
            case 6:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE7);
            break;
            case 7:
            GPIO_App_High(gpio_out_PCM5122_AUDIO_MUTE8);
            break;
            default :
            break;
        }    
    }
    */
}
void App_PCM5122_Init(void)
{
    uint8_t i;
    uint8_t pcmpage;
    for(i=0;i<8;i++) 
    {
        pcmpage=0x01;
        Pcm5122_IIC_Write(i,Pcm5122_PageSelect_Ctrl,&pcmpage,1);
        pcmpage=0x11;
        Pcm5122_IIC_Write(i,0x02,&pcmpage,0x01);
        pcmpage=0x00;
        Pcm5122_IIC_Write(i,Pcm5122_PageSelect_Ctrl,&pcmpage,1);
    }
}




long cmd_PCMAudioMute_output(int argc, char **argv)
{
    char *cmd = argv[0];
    unsigned char en;
	unsigned char out;
    out = atoi(argv[1]) & 0xFF;
	en = atoi(argv[2]) & 0xFF;
    PCM5122LogWarn("cmd=%s, out1=%d,en=%d;", cmd, out,en);
    App_PCM5122_SetVolumeMute(out,en);
}
MSH_CMD_EXPORT_ALIAS(cmd_PCMAudioMute_output,mute,out en);
long cmd_PCMVolumeValue_output(int argc, char **argv)
{
    char *cmd = argv[0];
    unsigned char value;
	unsigned char out;
    out = atoi(argv[1]) & 0xFF;
	value = atoi(argv[2]) & 0xFF;
    PCM5122LogWarn("cmd=%s, out1=%d,value=%d;", cmd, out,value);
    App_PCM5122_SetVolumeValue(out, value);
}
MSH_CMD_EXPORT_ALIAS(cmd_PCMVolumeValue_output,volume,value en);


#endif












